#
# Copyright (c) 2002, 2003 Art Haas
#
# This file is part of PythonCAD.
# 
# PythonCAD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# PythonCAD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PythonCAD; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# miscellaneous utility functions
#

from math import modf, pi

def tuple_to_two_floats(t):
    if not isinstance(t, tuple):
        raise TypeError, "Argument must be a tuple: " + `t`
    if len(t) != 2:
        raise ValueError, "Tuple must hold exactly two objects: " + `t`
    _obj1, _obj2 = t
    _x = _obj1
    if not isinstance(_x, float):
        _x = float(_obj1)
    _y = _obj2
    if not isinstance(_y, float):
        _y = float(_obj2)
    return (_x, _y)

def tuple_to_three_floats(t):
    if not isinstance(t, tuple):
        raise TypeError, "Argument must be a tuple: " + `t`
    if len(t) != 3:
        raise ValueError, "Tuple must hold exactly three objects: " + `t`
    _obj1, _obj2, _obj3 = t
    _x = _obj1
    if not isinstance(_x, float):
        _x = float(_obj1)
    _y = _obj2
    if not isinstance(_y, float):
        _y = float(_obj2)
    _z = _obj3
    if not isinstance(_z, float):
        _z = float(_obj3)
    return (_x, _y, _z)

def make_angle(angle):
    """Return an angle value such that -90 <= angle <= 90.

make_angle(angle)

The argument angle should be a float. Additionally the argument
is expected to be in degrees, not radians.
    """
    _a = angle
    if not isinstance(_a, float):
        _a = float(angle)
    _r, _i = modf(_a/360.0)
    _ta = _r * 360.0
    if _ta < -90.0:
        if _ta > -270.0:
            _a = _ta + 180.0
        else:
            _a = _ta + 360.0
    elif _ta > 90.0:
        if _ta < 270.0:
            _a = _ta - 180.0
        else:
            _a = _ta - 360.0
    else:
        _a = _ta
    return _a

def make_c_angle(angle):
    """Return an angle value such that 0 <= angle <= 360.

make_c_angle(angle)

The argument angle should be a float.
    """
    _a = angle
    if not isinstance(_a, float):
        _a = float(angle)
    if _a < 0.0:
        r, i = modf(_a/360.0)
        _a = (r * 360.0) + 360.0
    else:
        if _a > 360.0:
            r, i = modf(_a/360.0)
            _a = r * 360.0
    return _a

def degrees(value):
    """Convert a value from radians to degrees.

degrees()

In Python 2.3 this is available as the math.degrees() function, but
the value isn't scaled from -360.0 <= angle <= 360.0
    """
    if not isinstance(value, (int, float)):
        raise TypeError, "Invalid value: " + `value`
    _r, _i = modf(value/(2 * pi))
    return (_r * 360.0)

def radians(value):
    """Convert a value from degrees to radians.

radians()

In Python 2.3 this is available ad the math.radians() function, but
the value isn't scaled from -2*pi <= angle <= 2*pi
    """
    if not isinstance(value, (int, float)):
        raise TypeError, "Invalid value: " + `value`
    _r, _i = modf(value/360.0)
    return (_r * 2.0 * pi)
