#
# Copyright (c) 2002, 2003 Art Haas
#
# This file is part of PythonCAD.
# 
# PythonCAD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# PythonCAD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PythonCAD; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# menu functions for creating entities in drawing
# like segments, circles, etc ...
#

from math import hypot, pi, atan2

import gtk

import Generic.point as point
import Generic.segment as segment
import Generic.circle as circle
import Generic.arc as arc
import Generic.segjoint as segjoint
import Generic.leader as leader
import Generic.color as color


def make_tuple(text):
    try:
        tpl = eval(text)
    except:
        raise StandardError, "Invalid point: " + `text`
    if not isinstance(tpl, tuple):
        raise TypeError, "Invalid point: " + `tpl`
    if len(tpl) != 2:
        raise ValueError, "Invalid point: " + `tpl`
    return tpl

#
# points
#

def point_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    _redraw = False
    if _pt is None:
        _pt = point.Point(_x, _y)
        gtkimage.addObject(_pt)
        _redraw = True
    else:
        if _flag:
            gtkimage.addObject(_pt)
            _redraw = True
    if _redraw:
        gtkimage.redraw()

def point_entry_event_cb(gtkimage, widget, tool):
    entry = gtkimage.getEntry()
    text = entry.get_chars(0,-1)
    entry.delete_text(0,-1)
    if len(text):
        try:
            tpl = make_tuple(text)
        except:
            print "Invalid point: " + `text` # need better error handling ...
            return
        x, y = tpl
        p, flag = gtkimage.findPoint(x, y) # no need for tolerance checking ...
        redraw = False
        if p is None:
            p = point.Point(x, y)
            gtkimage.addObject(p)
            redraw = True
        else:
            if flag:
                gtkimage.addObject(p)
                redraw = True
        if redraw:
            gtkimage.redraw()

def point_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", point_button_press_cb)
    tool.setHandler("entry_event", point_entry_event_cb)

#
# segments
#

def segment_create(gtkimage, tool):
    assert len(tool) == 2, "Segment tool should hold two objects."
    active_layer = gtkimage.getActiveLayer()    
    _p2 = tool.popObject()
    if active_layer.findObject(_p2) is None:
        active_layer.addObject(_p2)
    _p1 = tool.popObject()
    if active_layer.findObject(_p1) is None:
        active_layer.addObject(_p1)
    _s = gtkimage.getOption('LINE_STYLE')
    _l = gtkimage.getOption('LINE_TYPE')
    _c = gtkimage.getOption('LINE_COLOR')
    _t = gtkimage.getOption('LINE_THICKNESS')
    gtkimage.addObject(segment.Segment(_p1, _p2, _s, _l, _c, _t))
    gtkimage.setPrompt("Click in the drawing area or enter a Point")    
    gtkimage.redraw()
    segment_mode_init(tool)

def segment_motion_notify_cb(gtkimage, widget, event, tool):
    _tx, _ty = tool.getCoordPair(0)
    _px, _py = gtkimage.coordToPixTransform(_tx, _ty)
    _gc = gtkimage.getGC()
    _x = event.x
    _y = event.y
    _cp = tool.getCurrentPoint()
    if _cp is not None:
        _xc, _yc = _cp
        widget.window.draw_line(_gc, _px, _py, _xc, _yc)
    tool.setCurrentPoint(_x, _y)
    widget.window.draw_line(_gc, _px, _py, _x, _y)

def segment_entry_event_cb(gtkimage, widget, tool):
    _entry = gtkimage.getEntry()
    _text = _entry.get_chars(0,-1)
    _entry.delete_text(0,-1)
    if len(_text):
        try:
            _tpl = make_tuple(_text)
        except:
            print "Invalid point: " + `_text` # need better error handling ...
            return
        _x, _y = _tpl
        _pt, _flag = gtkimage.findPoint(_x, _y) # no need for tol checking ...
        if _pt is None:
            _pt = point.Point(_x, _y)
        tool.pushObject(_pt)
        if len(tool) == 1:
            _x, _y = _pt.getCoords()
            tool.storeCoords(_x, _y)
            tool.setLocation(_x, _y)
            tool.setHandler("motion_notify", segment_motion_notify_cb)
            gtkimage.setPrompt("Enter the second Point or click in the drawing area")
            gtkimage.getGC().set_function(gtk.gdk.INVERT)
        else:
            segment_create(gtkimage, tool)
            
def segment_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    if _pt is None:
        _pt = point.Point(_x, _y)
    tool.pushObject(_pt)
    if len(tool) == 1:
        _x, _y = _pt.getCoords()
        tool.storeCoords(_x, _y)
        tool.setLocation(_x, _y)
        tool.setHandler("motion_notify", segment_motion_notify_cb)
        gtkimage.setPrompt("Enter the second point or click in the drawing area")
        gtkimage.getGC().set_function(gtk.gdk.INVERT)
    else:
        segment_create(gtkimage, tool)

def segment_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", segment_button_press_cb)
    tool.setHandler("entry_event", segment_entry_event_cb)

#
# rectangles
#

def make_point(gtkimage, x, y):
    _pt = point.Point(x, y)
    _active_layer = gtkimage.getActiveLayer()
    _lp = _active_layer.findObject(_pt)
    if _lp is None:
        _active_layer.addObject(_pt)
    else:
        _pt = _lp
    return _pt
    
def rectangle_create(gtkimage, tool):
    assert len(tool) == 2, "Segment tool should hold two objects."
    t2 = tool.popObject()
    t1 = tool.popObject()
    pts = []
    try:
        pt = make_point(gtkimage, t1[0], t1[1])
        pts.append(pt)
        pt = make_point(gtkimage, t1[0], t2[1])
        pts.append(pt)
        pt = make_point(gtkimage, t2[0], t1[1])
        pts.append(pt)
        pt = make_point(gtkimage, t2[0], t2[1])
    except:
        print "Failed to make rectangle endpoints" # needs better handling
        return
    pts.append(pt)
    _s = gtkimage.getOption('LINE_STYLE')
    _l = gtkimage.getOption('LINE_TYPE')
    _c = gtkimage.getOption('LINE_COLOR')
    _t = gtkimage.getOption('LINE_THICKNESS')

    try:
        gtkimage.addObject(segment.Segment(pts[0], pts[1], _s, _l, _c, _t))
        gtkimage.addObject(segment.Segment(pts[0], pts[2], _s, _l, _c, _t))
        gtkimage.addObject(segment.Segment(pts[1], pts[3], _s, _l, _c, _t))
        gtkimage.addObject(segment.Segment(pts[2], pts[3], _s, _l, _c, _t))
    except:
        print "Failed to add rectangle segments!" # needs better error handling
        return
    gtkimage.redraw()
    gtkimage.setPrompt("Click in the drawing area or enter a Point")
    rectangle_mode_init(tool)

def rectangle_motion_notify_cb(gtkimage, widget, event, tool):
    _tx, _ty = tool.getLocation()
    _px, _py = gtkimage.coordToPixTransform(_tx, _ty)
    _width, _height = gtkimage.getSize()
    _gc = gtkimage.getGC()
    _x = event.x
    _y = event.y
    _cp = tool.getCurrentPoint()
    if _cp is not None:
        _xc, _yc = _cp
        _xmin = min(_xc, _px)
        _ymin = min(_yc, _py)
        _rw = abs(_xc - _px)
        _rh = abs(_yc - _py)
        widget.window.draw_rectangle(_gc, gtk.FALSE, _xmin, _ymin, _rw, _rh)
    tool.setCurrentPoint(_x, _y)
    _xmin = min(_x, _px)
    _ymin = min(_y, _py)
    _rw = abs(_x - _px)
    _rh = abs(_y - _py)
    widget.window.draw_rectangle(_gc, gtk.FALSE, _xmin, _ymin, _rw, _rh)

def rectangle_entry_event_cb(gtkimage, widget, tool):
    entry = gtkimage.getEntry()
    text = entry.get_chars(0,-1)
    entry.delete_text(0,-1)
    if len(text):
        try:
            tpl = make_tuple(text)
        except:
            print "Invalid point: " + `text`
            return
        x, y = tpl
        tool.pushObject(tpl)
        if len(tool) == 1:
            tool.setLocation(x, y)
            tool.setHandler("motion_notify", rectangle_motion_notify_cb)
            gtkimage.setPrompt("Enter the second Point or click in the drawing area")
            gtkimage.getGC().set_function(gtk.gdk.INVERT)
        else:
            rectangle_create(gtkimage, tool)
            
def rectangle_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    if _pt is not None:
        _x, _y = _pt.getCoords()
    tool.pushObject((_x, _y))
    if len(tool) == 1:
        tool.storeCoords(_x, _y)
        tool.setLocation(_x, _y)
        tool.setHandler("motion_notify", rectangle_motion_notify_cb)
        gtkimage.setPrompt("Enter the second point or click in the drawing area")
        gtkimage.getGC().set_function(gtk.gdk.INVERT)
    else:
        rectangle_create(gtkimage, tool)

def rectangle_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", rectangle_button_press_cb)
    tool.setHandler("entry_event", rectangle_entry_event_cb)
    
#
# circles
#

def circle_create(gtkimage, tool):
    assert len(tool) == 2, "Circle tool should hold two objects."
    obj = tool.popObject()
    cen = tool.popObject()
    if isinstance(obj, point.Point):
        rad = cen - obj
    else:
        rad = obj
    active_layer = gtkimage.getActiveLayer()
    if active_layer.findObject(cen) is None:
        active_layer.addObject(cen)
    _s = gtkimage.getOption('LINE_STYLE')
    _l = gtkimage.getOption('LINE_TYPE')
    _c = gtkimage.getOption('LINE_COLOR')
    _t = gtkimage.getOption('LINE_THICKNESS')
    gtkimage.addObject(circle.Circle(cen, rad, _s, _l, _c, _t))
    gtkimage.setPrompt("Click in the drawing area or enter a Point")
    gtkimage.redraw()
    init_func = tool.getHandler("initialize")
    init_func(tool)

def circle_make_point(gtkimage, text, addflag=True):
    _tpl = make_tuple(text)
    _x, _y = _tpl
    _pt, _flag = gtkimage.findPoint(_x, _y)
    if _pt is None:
        _pt = point.Point(_x, _y)
    return _pt

def circle_make_radius(text):
    try:
        rad = eval(text)
    except:
        raise ValueError, "Invalid radius: " + `text`
    _rad = rad
    if not isinstance(_rad, float):
        _rad = float(rad)
    if _rad < 1e-10:
        raise ValueError, "Invalid radius: " + `_rad`
    return _rad

def circle_center_motion_notify_cb(gtkimage, widget, event, tool):
    _tx, _ty = tool.getLocation()
    _px, _py = gtkimage.coordToPixTransform(_tx, _ty)
    _width, _height = gtkimage.getSize()
    _gc = gtkimage.getGC()
    _x = event.x
    _y = event.y
    _cp = tool.getCurrentPoint()
    if _cp is not None:
        _xc, _yc = _cp
        _radius = hypot((_px - _xc),(_py - _yc))
        _xmin = _px - _radius
        _ymin = _py - _radius
        _cw = _ch = _radius * 2
        widget.window.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                               0, 360*64)
    tool.setCurrentPoint(_x, _y)
    _radius = hypot((_px - _x),(_py - _y))
    _xmin = _px - _radius
    _ymin = _py - _radius
    _cw = _ch = _radius * 2
    widget.window.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                           0, 360*64)

def circle_radius_entry_event_cb(gtkimage, widget, tool):
    entry = gtkimage.getEntry()
    text = entry.get_chars(0,-1)
    entry.delete_text(0,-1)
    if len(text):
        try:
            radius = circle_make_radius(text)
        except:
            print "Invalid radius: " + `text` # better error handling needed
            return
        tool.pushObject(radius)
        circle_create(gtkimage, tool)

def circle_point_entry_event_cb(gtkimage, widget, tool):
    entry = gtkimage.getEntry()
    text = entry.get_chars(0,-1)
    entry.delete_text(0,-1)
    if len(text):
        try:
            p = circle_make_point(gtkimage, text)
        except:
            print "Invalid point: " + `text` # better error handling needed
            return
        tool.pushObject(p)
        if len(tool) == 1:
            x, y = p.getCoords()
            tool.setLocation(x, y)
            tool.setHandler("motion_notify", circle_center_motion_notify_cb)
            tool.setHandler("entry_event", circle_radius_entry_event_cb)
            gtkimage.setPrompt("Enter the radius or click in the drawing area")
            gtkimage.getGC().set_function(gtk.gdk.INVERT)
        else:
            circle_create(gtkimage, tool)
    
def circle_center_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    if _pt is None:
        _pt = point.Point(_x, _y)
    tool.pushObject(_pt)
    if len(tool) == 1:
        _x, _y = _pt.getCoords()
        tool.storeCoords(_x, _y)
        tool.setLocation(_x, _y)
        tool.setHandler("motion_notify", circle_center_motion_notify_cb)
        tool.setHandler("entry_event", circle_radius_entry_event_cb)
        gtkimage.setPrompt("Enter the radius or click in the drawing area")
        gtkimage.getGC().set_function(gtk.gdk.INVERT)
    else:
        circle_create(gtkimage, tool)

def circle_center_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", circle_center_button_press_cb)
    tool.setHandler("entry_event", circle_point_entry_event_cb)
    tool.setHandler("initialize", circle_center_mode_init)

#
# circle from two points
#

def circle_tp_create(gtkimage, tool):
    assert len(tool) == 2, "Circle tool should hold two objects."
    _p2 = tool.popObject()
    _p1 = tool.popObject()
    _xc = (_p1.x + _p2.x)/2.0
    _yc = (_p1.y + _p2.y)/2.0
    _rad = (_p2 - _p1)/2.0
    _cp = point.Point(_xc, _yc)
    _active_layer = gtkimage.getActiveLayer()
    _lp = _active_layer.findObject(_cp)
    if _lp is None:
        _active_layer.addObject(_cp)
        _lp = _cp
    _s = gtkimage.getOption('LINE_STYLE')
    _l = gtkimage.getOption('LINE_TYPE')
    _c = gtkimage.getOption('LINE_COLOR')
    _t = gtkimage.getOption('LINE_THICKNESS')
    gtkimage.addObject(circle.Circle(_lp, _rad, _s, _l, _c, _t))
    gtkimage.redraw()
    gtkimage.setPrompt("Click in the drawing area or enter a Point")
    init_func = tool.getHandler("initialize")
    init_func(tool)

def circle_tp_motion_notify_cb(gtkimage, widget, event, tool):
    tx, ty = tool.getLocation()
    px, py = gtkimage.coordToPixTransform(tx, ty)
    width, height = gtkimage.getSize()
    gc = gtkimage.getGC()
    x = event.x
    y = event.y
    cp = tool.getCurrentPoint()
    if cp is not None:
        xc, yc = cp
        xcen = px + (xc - px)//2
        ycen = py + (yc - py)//2
        radius = hypot((xc - xcen),(yc - ycen))
        xmin = xcen - radius
        ymin = ycen - radius
        cw = ch = radius * 2
        widget.window.draw_arc(gc, gtk.FALSE, xmin, ymin, cw, ch,
                               0, 360*64)
    tool.setCurrentPoint(x, y)
    xcen = px + (x - px)//2
    ycen = py + (y - py)//2
    radius = hypot((x - xcen),(y - ycen))
    xmin = xcen - radius
    ymin = ycen - radius
    cw = ch = radius * 2
    widget.window.draw_arc(gc, gtk.FALSE, xmin, ymin, cw, ch,
                           0, 360*64)

def circle_tp_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    if _pt is None:
        _pt = point.Point(_x, _y)
    tool.pushObject(_pt)
    if len(tool) == 1:
        _x, _y = _pt.getCoords()
        tool.storeCoords(_x, _y)
        tool.setLocation(_x, _y)
        tool.setHandler("motion_notify", circle_tp_motion_notify_cb)
        gtkimage.setPrompt("Enter the second point or click in the drawing area")
        gtkimage.getGC().set_function(gtk.gdk.INVERT)
    else:
        circle_tp_create(gtkimage, tool)

def circle_tp_entry_event_cb(gtkimage, widget, tool):
    entry = gtkimage.getEntry()
    text = entry.get_chars(0,-1)
    entry.delete_text(0,-1)
    if len(text):
        try:
            p = circle_make_point(gtkimage, text, False)
        except:
            print "Invalid point: " + `text` # need better error handler
            return
        tool.pushObject(p)
        if len(tool) == 1:
            gtkimage.addObject(p)
            x, y = p.getCoords()
            tool.setLocation(x, y)
            tool.setHandler("motion_notify", circle_tp_motion_notify_cb)
            gtkimage.setPrompt("Enter another point or click in the drawing area")
            gtkimage.getGC().set_function(gtk.gdk.INVERT)
        else:
            circle_tp_create(gtkimage, tool)

def circle_tp_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", circle_tp_button_press_cb)
    tool.setHandler("entry_event", circle_tp_entry_event_cb)
    tool.setHandler("initialize", circle_tp_mode_init)

#
# arcs
#

def arc_make_angle(text):
    ang = eval(text)
    _ang = ang
    if not isinstance(_ang, float):
        _ang = float(ang)
    return _ang

def arc_center_create(gtkimage, tool):
    assert len(tool) == 4, "Arc tool should hold four objects."
    ea = tool.popObject()
    sa = tool.popObject()
    rad = tool.popObject()
    cen = tool.popObject()
    _s = gtkimage.getOption('LINE_STYLE')
    _l = gtkimage.getOption('LINE_TYPE')
    _c = gtkimage.getOption('LINE_COLOR')
    _t = gtkimage.getOption('LINE_THICKNESS')
    active_layer = gtkimage.getActiveLayer()
    if active_layer.findObject(cen) is None:
        active_layer.addObject(cen)
    new_arc = arc.Arc(cen, rad, sa, ea, _s, _l, _c, _t)
    gtkimage.addObject(new_arc)
    gtkimage.redraw()
    gtkimage.setPrompt("Click in the drawing area or enter a Point")
    init_func = tool.getHandler("initialize")
    init_func(tool)

def arc_center_end_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    _xc, _yc = tool.getCoordPair(0)
    if _pt is not None:
        _x, _y = _pt.getCoords()
    _end_angle = (180.0/pi) * atan2((_y - _yc),(_x - _xc))
    tool.pushObject(_end_angle)
    arc_center_create(gtkimage, tool)

def arc_angle_motion_notify_cb(gtkimage, widget, event, tool):
    _px, _py = gtkimage.getPoint()
    _cx, _cy = tool.getCoordPair(0) # arc center
    _pcx, _pcy = gtkimage.coordToPixTransform(_cx, _cy)
    _rx, _ry = tool.getCoordPair(1) # radius point
    _prx, _pry = gtkimage.coordToPixTransform(_rx, _ry)
    _rad = hypot((_rx - _cx), (_ry - _cy)) # radius
    _sa = (180.0/pi) * atan2((_ry - _cy), (_rx - _cx)) # start angle
    if _sa < 0.0:
        _sa = _sa + 360.0
    _prad = int(_rad/gtkimage.getUnitsPerPixel())
    _xmin = _pcx - _prad
    _ymin = _pcy - _prad
    _cw = _ch = _prad * 2
    _gc = gtkimage.getGC()
    _x = event.x
    _y = event.y
    _cp = tool.getCurrentPoint()
    _win = widget.window
    if _cp is not None:
        _tx, _ty = tool.getLocation()
        _angle = (180.0/pi) * atan2((_ty - _cy), (_tx - _cx))
        if _angle < 0.0:
            _angle = _angle + 360.0
        if _sa < _angle:
            _sweep = _angle - _sa
            _win.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                          (_sa * 64), (_sweep * 64))
        else:
            _sweep = 360.0 - _sa
            _win.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                          (_sa * 64), (_sweep * 64))
            _win.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                          0, (_angle * 64))
    tool.setCurrentPoint(_x, _y)
    tool.setLocation(_px, _py)
    _angle = (180.0/pi) * atan2((_py - _cy), (_px - _cx))
    if _angle < 0.0:
        _angle = _angle + 360.0
    if _sa < _angle:
        _sweep = _angle - _sa
        _win.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                      (_sa * 64), (_sweep * 64))
    else:
        _sweep = 360.0 - _sa
        _win.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                      (_sa * 64), (_sweep * 64))
        _win.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                      0, (_angle * 64))
    
def arc_center_ea_entry_event_cb(gtkimage, widget, tool):
    entry = gtkimage.getEntry()
    text = entry.get_chars(0,-1)
    entry.delete_text(0,-1)
    if len(text):
        try:
            ea = arc_make_angle(text)
        except:
            print "Invalid angle: " + `text` # need better error handling
            return
        tool.pushObject(ea)
        arc_center_create(gtkimage, tool)

def arc_center_sa_entry_event_cb(gtkimage, widget, tool):
    entry = gtkimage.getEntry()
    text = entry.get_chars(0,-1)
    entry.delete_text(0,-1)
    if len(text):
        try:
            sa = arc_make_angle(text)
        except:
            print "Invalid angle: " + `text` # need better error handling
            return
        tool.pushObject(sa)
        tool.setHandler("entry_event", arc_center_ea_entry_event_cb)
        gtkimage.setPrompt("Enter the end angle of the arc.")

def arc_center_radius_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    _xc, _yc = tool.getCoordPair(0)
    if _pt is not None:
        _x, _y = _pt.getCoords()
    _radius = hypot((_x - _xc), (_y - _yc))
    _start_angle = (180.0/pi) * atan2((_y - _yc), (_x - _xc))
    if _start_angle < 0.0:
        _start_angle = _start_angle + 360.0
    tool.storeCoords(_x, _y)
    tool.setPoint(1, _x, _y)
    tool.pushObject(_radius)
    tool.pushObject(_start_angle)
    tool.setLocation(_x, _y)
    tool.delHandler("entry_event")
    tool.setHandler("motion_notify", arc_angle_motion_notify_cb)
    tool.clearCurrentPoint()
    gtkimage.refresh()
    gtkimage.getGC().set_function(gtk.gdk.INVERT)
    tool.setHandler("button_press", arc_center_end_button_press_cb)
    gtkimage.setPrompt("Click in the drawing area to finish the arc")
    
def arc_center_radius_entry_cb(gtkimage, widget, tool):
    entry = gtkimage.getEntry()
    text = entry.get_chars(0,-1)
    entry.delete_text(0,-1)
    if len(text):
        try:
            radius = circle_make_radius(text)
        except:
            print "Invalid radius: " + `text` # need better error handling
            return
        tool.pushObject(radius)
        tool.setHandler("entry_event", arc_center_sa_entry_event_cb)
        tool.delHandler("button_press")
        gtkimage.setPrompt("Enter the start angle of the arc.")

def arc_radius_motion_notify_cb(gtkimage, widget, event, tool):
    _cx, _cy = tool.getCoordPair(0)
    _px, _py = gtkimage.coordToPixTransform(_cx, _cy)
    _gc = gtkimage.getGC()
    _x = event.x
    _y = event.y
    _cp = tool.getCurrentPoint()
    if _cp is not None:
        _xc, _yc = _cp
        _radius = hypot((_px - _xc),(_py - _yc))
        _xmin = _px - _radius
        _ymin = _py - _radius
        _cw = _ch = _radius * 2
        widget.window.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                               0, 360*64)
    tool.setCurrentPoint(_x, _y)
    _radius = hypot((_px - _x),(_py - _y))
    _xmin = _px - _radius
    _ymin = _py - _radius
    _cw = _ch = _radius * 2
    widget.window.draw_arc(_gc, gtk.FALSE, _xmin, _ymin, _cw, _ch,
                           0, 360*64)

def arc_center_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    if _pt is None:
        _pt = point.Point(_x, _y)
    _x, _y = _pt.getCoords()
    tool.pushObject(_pt)
    tool.storeCoords(_x, _y)
    tool.setPoint(0, _x, _y)
    tool.setLocation(_x, _y)
    gtkimage.getGC().set_function(gtk.gdk.INVERT)
    tool.setHandler("motion_notify", arc_radius_motion_notify_cb)
    tool.setHandler("entry_event", arc_center_radius_entry_cb)
    tool.setHandler("button_press", arc_center_radius_button_press_cb)
    gtkimage.setPrompt("Enter the radius or click in the drawing area")

def arc_center_entry_event_cb(gtkimage, widget, tool):
    _entry = gtkimage.getEntry()
    _text = _entry.get_chars(0,-1)
    _entry.delete_text(0,-1)
    if len(_text):
        try:
            _p = circle_make_point(gtkimage, _text, False)
        except:
            print "Invalid point: " + `_text` # need better error handling
            return
        tool.pushObject(_p)
        _x, _y = _p.getCoords()
        tool.storeCoords(_x, _y)
        tool.setPoint(0, _x, _y)
        tool.setLocation(_x, _y)
        tool.setHandler("motion_notify", arc_radius_motion_notify_cb)
        gtkimage.getGC().set_function(gtk.gdk.INVERT)
        tool.setHandler("button_press", arc_center_radius_button_press_cb)
        tool.setHandler("entry_event", arc_center_radius_entry_cb)
        gtkimage.setPrompt("Enter the arc radius or click in the drawing area")

def arc_center_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", arc_center_button_press_cb)
    tool.setHandler("entry_event", arc_center_entry_event_cb)
    tool.setHandler("initialize", arc_center_mode_init)

#
# chamfers
#

def chamfer_button_press_cb(gtkimage, widget, event, tool):
    print "in chamfer_button_press_cb()"
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _new_pt = gtkimage.findPoint(_x, _y, _tol)
    if _pt is not None and not _new_pt:
        _active_layer = gtkimage.getActiveLayer()
        _users = _active_layer.usesObject(_pt)
        if ((len(_users) == 2) and
            (isinstance(_users[0], segment.Segment)) and
            (isinstance(_users[1], segment.Segment))):
            _s1 = _users[0]
            _s2 = _users[1]
            _len = gtkimage.getOption('CHAMFER_LENGTH')
            _s = gtkimage.getOption('LINE_STYLE')
            _l = gtkimage.getOption('LINE_TYPE')
            _c = gtkimage.getOption('LINE_COLOR')
            _t = gtkimage.getOption('LINE_THICKNESS')
            #
            # the following is a work-around that needs to
            # eventually be replaced ...
            #
            _p1, _p2 = _s1.getEndpoints()
            _x1, _y1 = _p1.getCoords()
            _x2, _y2 = _p2.getCoords()
            _slen = _s1.length()
            _factor = 2.0
            _r = (_slen - (_len/_factor))/_slen
            # print "r: %g" % _r
            _xn = _x1 + _r * (_x2 - _x1)
            _yn = _y1 + _r * (_x2 - _x1)
            _pc = _active_layer.hasPoint((_xn, _yn))
            while _pc is not None:
                _factor = _factor + 1.0
                if _factor > 1000:
                    break
                _r = (_slen - (_len/_factor))/_slen
                _xn = _x1 + _r * (_x2 - _x1)
                _yn = _y1 + _r * (_x2 - _x1)
                _pc = _active_layer.hasPoint((_xn, _yn))
            if _pc is None:
                # print "xn: %g" % _xn
                # print "yn: %g" % _yn
                _pc = point.Point(_xn, _yn)
                _active_layer.addObject(_pc)
                if _pt is _p1:
                    _s1.setP1(_pc)
                else:
                    _s1.setP2(_pc)
                _active_layer.update()
                _ptx, _pty = _pt.getCoords()
                _pc.setx(_ptx)
                _pc.sety(_pty)
                _chamfer = segjoint.Chamfer(_s1, _s2, _len, _s, _l, _c, _t)
                _active_layer.addObject(_chamfer)
                gtkimage.redraw()

def chamfer_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", chamfer_button_press_cb)
    
#
# fillet
#

def fillet_button_press_cb(gtkimage, widget, event, tool):
    print "in fillet_button_press_cb()"
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _new_pt = gtkimage.findPoint(_x, _y, _tol)
    if _pt is not None and not _new_pt:
        _active_layer = gtkimage.getActiveLayer()
        _users = _active_layer.usesObject(_pt)
        if ((len(_users) == 2) and
            (isinstance(_users[0], segment.Segment)) and
            (isinstance(_users[1], segment.Segment))):
            _s1 = _users[0]
            _s2 = _users[1]
            _rad = gtkimage.getOption('FILLET_RADIUS')
            _s = gtkimage.getOption('LINE_STYLE')
            _l = gtkimage.getOption('LINE_TYPE')
            _c = gtkimage.getOption('LINE_COLOR')
            _t = gtkimage.getOption('LINE_THICKNESS')
            #
            # the following is a work-around that needs to
            # eventually be replaced ...
            #
            _p1, _p2 = _s1.getEndpoints()
            _x1, _y1 = _p1.getCoords()
            _x2, _y2 = _p2.getCoords()
            _slen = _s1.length()
            _factor = 2.0
            _r = 1.0 - (_slen/_factor)
            _xn = _x1 + _r * (_x2 - _x1)
            _yn = _y1 + _r * (_x2 - _x1)
            _pc = _active_layer.hasPoint((_xn, _yn))
            while _pc is not None:
                _factor = _factor + 1.0
                if _factor > 1000:
                    break
                _r = 1.0 - (_slen/_factor)
                _xn = _x1 + _r * (_x2 - _x1)
                _yn = _y1 + _r * (_x2 - _x1)
                _pc = _active_layer.hasPoint((_xn, _yn))
            if _pc is None:
                # print "xn: %g" % _xn
                # print "yn: %g" % _yn
                _pc = point.Point(_xn, _yn)
                _active_layer.addObject(_pc)
                if _pt is _p1:
                    _s1.setP1(_pc)
                else:
                    _s1.setP2(_pc)
                _active_layer.update()
                _ptx, _pty = _pt.getCoords()
                _pc.setx(_ptx)
                _pc.sety(_pty)
                _fillet = segjoint.Fillet(_s1, _s2, _rad, _s, _l, _c, _t)
                _active_layer.addObject(_fillet)
                gtkimage.redraw()

def fillet_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", fillet_button_press_cb)

#
# leader lines
#

def leader_create(gtkimage, tool):
    assert len(tool) == 3, "Leader tool should hold three objects."
    _active_layer = gtkimage.getActiveLayer()
    _p3 = tool.popObject()
    if _active_layer.findObject(_p3) is None:
        _active_layer.addObject(_p3)
    _p2 = tool.popObject()
    if _active_layer.findObject(_p2) is None:
        _active_layer.addObject(_p2)
    _p1 = tool.popObject()
    if _active_layer.findObject(_p1) is None:
        _active_layer.addObject(_p1)
    _size = 1.0 # fixme
    _s = gtkimage.getOption('LINE_STYLE')
    _l = gtkimage.getOption('LINE_TYPE')
    _c = gtkimage.getOption('LINE_COLOR')
    _t = gtkimage.getOption('LINE_THICKNESS')
    gtkimage.addObject(leader.Leader(_p1, _p2, _p3, _size, _s, _l, _c, _t))
    gtkimage.setPrompt("Click in the drawing area to enter the first point")
    gtkimage.redraw()
    leader_mode_init(tool)
    
def leader_motion_notify_cb(gtkimage, widget, event, tool):
    _idx = 0
    if len(tool) == 2:
        _idx = 1
    _tx, _ty = tool.getCoordPair(_idx)
    _px, _py = gtkimage.coordToPixTransform(_tx, _ty)
    _gc = gtkimage.getGC()
    _x = event.x
    _y = event.y
    _cp = tool.getCurrentPoint()
    if _cp is not None:
        _xc, _yc = _cp
        widget.window.draw_line(_gc, _px, _py, _xc, _yc)
    tool.setCurrentPoint(_x, _y)
    widget.window.draw_line(_gc, _px, _py, _x, _y)

def leader_final_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    if _pt is None:
        _pt = point.Point(_x, _y)
    else:
        _x, _y = _pt.getCoords()
    tool.pushObject(_pt)
    tool.storeCoords(_x, _y)
    tool.setPoint(2, _x, _y)
    tool.setLocation(_x, _y)
    leader_create(gtkimage, tool)
    gtkimage.setPrompt("Click in the drawing area to place the initial point")
    
def leader_second_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    if _pt is None:
        _pt = point.Point(_x, _y)
    else:
        _x, _y = _pt.getCoords()
    tool.pushObject(_pt)
    tool.storeCoords(_x, _y)
    tool.setPoint(1, _x, _y)
    tool.setLocation(_x, _y)
    tool.clearCurrentPoint()
    # gtkimage.getGC().set_function(gtk.gdk.INVERT)
    # tool.setHandler("motion_notify", leader_motion_notify_cb)
    tool.setHandler("button_press", leader_final_button_press_cb)    
    gtkimage.setPrompt("Click in the drawing area to place the final point")
    
def leader_first_button_press_cb(gtkimage, widget, event, tool):
    _x, _y = gtkimage.getPoint()
    _tol = gtkimage.getTolerance()
    _pt, _flag = gtkimage.findPoint(_x, _y, _tol)
    if _pt is None:
        _pt = point.Point(_x, _y)
    else:
        _x, _y = _pt.getCoords()
    tool.pushObject(_pt)
    tool.storeCoords(_x, _y)
    tool.setPoint(0, _x, _y)
    tool.setLocation(_x, _y)
    gtkimage.getGC().set_function(gtk.gdk.INVERT)
    tool.setHandler("motion_notify", leader_motion_notify_cb)
    tool.setHandler("button_press", leader_second_button_press_cb)
    gtkimage.setPrompt("Click in the drawing area to place the second point")

def leader_mode_init(tool):
    tool.initialize()
    tool.setHandler("button_press", leader_first_button_press_cb)
#
# set the active style
#

def set_active_style(gtkimage):
    window = gtkimage.getWindow()
    dialog = gtk.Dialog("Set Active Style", window,
                        gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_OK, gtk.RESPONSE_OK,
                        gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
    hbox = gtk.HBox(gtk.FALSE, 2)
    hbox.set_border_width(2)
    dialog.vbox.pack_start(hbox, gtk.FALSE, gtk.FALSE, 0)
    label = gtk.Label("Style:")
    cur_style = gtkimage.getOption('LINE_STYLE')
    styles = gtkimage.getAllStyles()
    menu = gtk.Menu()
    idx = 0
    for i in xrange(len(styles)):
        s = styles[i]
        if s is cur_style:
            idx = i
        item = gtk.MenuItem(s.getName())
        menu.append(item)
    opt_menu = gtk.OptionMenu()
    opt_menu.set_menu(menu)
    opt_menu.set_history(idx)
    hbox.pack_start(label, gtk.FALSE, gtk.FALSE, 0)
    hbox.pack_start(opt_menu, gtk.TRUE, gtk.TRUE, 0)
    dialog.show_all()
    response = dialog.run()
    if response == gtk.RESPONSE_OK:
        s = styles[opt_menu.get_history()]
        gtkimage.setOption('LINE_STYLE', s)
    dialog.destroy()

#
# set the current color
#

def set_active_color(gtkimage):
    window = gtkimage.getWindow()
    dialog = gtk.ColorSelectionDialog("Set Active Color")
    dialog.set_transient_for(window)
    colorsel = dialog.colorsel

    prev_color = gtkimage.getOption('LINE_COLOR')
    if gtkimage.hasGTKColor(prev_color):
        gtk_color = gtkimage.getGTKColor(prev_color)
    else:
        da = gtkimage.getDA()
        cstring = str(prev_color)
        gtk_color = da.get_colormap().alloc_color(cstring)
        gtkimage.saveGTKColor(prev_color, gtk_color)
    colorsel.set_previous_color(gtk_color)
    colorsel.set_current_color(gtk_color)
    colorsel.set_has_palette(gtk.TRUE)
    response = dialog.run()
    if response == gtk.RESPONSE_OK:
        gtk_color = colorsel.get_current_color()
        red = int((gtk_color.red/65535.0)*255)
        green = int((gtk_color.green/65535.0)*255)
        blue = int((gtk_color.blue/65535.0)*255)
        col = color.Color(red, green, blue)
        gtkimage.setOption('LINE_COLOR', col)
    dialog.destroy()

#
# set the current linetype
#

def set_active_linetype(gtkimage):
    window = gtkimage.getWindow()
    dialog = gtk.Dialog("Set Active Linetype", window,
                        gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                        (gtk.STOCK_OK, gtk.RESPONSE_OK,
                        gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
    hbox = gtk.HBox(gtk.FALSE, 2)
    hbox.set_border_width(2)
    dialog.vbox.pack_start(hbox, gtk.FALSE, gtk.FALSE, 0)
    
    label = gtk.Label("Linetype:")
    hbox.pack_start(label, gtk.FALSE, gtk.FALSE, 0)
    cur_linetype = gtkimage.getOption('LINE_TYPE')
    linetypes = gtkimage.getAllLinetypes()
    menu = gtk.Menu()
    idx = 0
    for i in xrange(len(linetypes)):
        lt = linetypes[i]
        if lt is cur_linetype:
            idx = i
        item = gtk.MenuItem(lt.getName())
        menu.append(item)
    opt_menu = gtk.OptionMenu()
    opt_menu.set_menu(menu)
    opt_menu.set_history(idx)
    hbox.pack_start(opt_menu, gtk.TRUE, gtk.TRUE, 0)
    dialog.show_all()
    response = dialog.run()
    if response == gtk.RESPONSE_OK:
        lt = linetypes[opt_menu.get_history()]
        gtkimage.setOption('LINE_TYPE', lt)
        
    dialog.destroy()

#
# set the current line thickness
#

def set_line_thickness(gtkimage):
    _window = gtkimage.getWindow()
    _dialog = gtk.Dialog("Set Line Thickness", _window,
                         gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT,
                         (gtk.STOCK_OK,
                          gtk.RESPONSE_OK,
                          gtk.STOCK_CANCEL,
                          gtk.RESPONSE_CANCEL))
    _hbox = gtk.HBox(gtk.FALSE, 2)
    _hbox.set_border_width(2)
    _dialog.vbox.pack_start(_hbox, gtk.FALSE, gtk.FALSE, 0)
    _label = gtk.Label("Thickness:")
    _hbox.pack_start(_label, gtk.FALSE, gtk.FALSE, 0)
    _thickness = gtkimage.getOption('LINE_THICKNESS')
    _adj = gtk.Adjustment(_thickness, 0.0001, 20.0, 0.1, 1.0, 1.0)
    _sb = gtk.SpinButton(_adj)
    _sb.set_digits(2)
    _sb.set_numeric(gtk.FALSE)
    _hbox.pack_start(_sb, gtk.TRUE, gtk.TRUE, 0)
    _dialog.show_all()
    _response = _dialog.run()
    if _response == gtk.RESPONSE_OK:
        _size = _sb.get_value()
        gtkimage.setOption('LINE_THICKNESS', _size)
    _dialog.destroy()
