#!/usr/bin/env python

#
# Copyright (c) 2002, 2003 Art Haas
#
# This file is part of PythonCAD.
# 
# PythonCAD is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# PythonCAD is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PythonCAD; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
# main routine to start GTK-based pycad
#

import getopt
import sys
import os
import gtk

import Generic.globals
import Interface.Gtk.gtkimage
import Generic.image
import Generic.imageio
import Generic.fileio
import Generic.selections
import Generic.preferences

#
# set up global variables ...
#

Generic.globals.imagelist = []

Generic.globals.selectobj = Generic.selections.Selection()

#
# routines to store/remove references to the opened drawings
#

def add_image(gtkimage):
    """Store a reference to a loaded image.

add_image(gtkimage)
    """
    if not isinstance(gtkimage, Interface.Gtk.gtkimage.GTKImage):
        raise TypeError, "Invalid image: "+ `gtkimage`
    Generic.globals.imagelist.append(gtkimage)

def del_image(gtkimage):
    """Remove a reference to a loaded image.

del_image(gtkimage)
    """
    if not isinstance(gtkimage, Interface.Gtk.gtkimage.GTKImage):
        raise TypeError, "Invalid image: " + `gtkimage`
    for _i in xrange(len(Generic.globals.imagelist)):
        _gimage = Generic.globals.imagelist[_i]
        if gtkimage is _gimage:
            del Generic.globals.imagelist[_i]
            _gimage.window.destroy()
            if not len(Generic.globals.imagelist):
                gtk.main_quit()
            break
    
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "h", ["help"])
    except getopt.GetoptError, e:
        sys.stderr.write("Error: %s\n" % e)
        sys.exit(1)
    for o, a in opts:
        if o in ("-h", "--help"):
            sys.stdout.write("No help yet! Sorry ...\n")
            sys.exit()
    #
    # load up global and user preferences
    #

    Generic.preferences.initialize_prefs()
    Generic.preferences.load_global_prefs()
    _user_flag = Generic.globals.prefs['USER_PREFS']
    if _user_flag:
        Generic.preferences.load_user_prefs()
        
    #
    #
    # 
    for arg in args:
        if os.path.exists(arg):
            sys.stdout.write("Opening '%s'\n" % arg)
            try:
                imfile = Generic.fileio.CompFile(arg, "r")
            except IOError, e:
                sys.stderr.write("Can't open '%s'! %s" % (arg, e))
                continue
            img = Interface.Gtk.gtkimage.GTKImage()
            try:
                Generic.imageio.load_image(img, imfile)
                fname = os.path.realpath(arg)
                img.setFilename(fname)
                img.getWindow().set_title(os.path.basename(fname))
                add_image(img)
                img.window.show_all()
                img.fitImage()
            finally:
                imfile.close()
        else:
            sys.stderr.write("Can't find file '%s' - Skipping ...\n" % arg)
    if not len(Generic.globals.imagelist):
        img = Interface.Gtk.gtkimage.GTKImage()
        add_image(img)
        img.window.show_all()
    gtk.main()

if __name__ == '__main__':
    _major, _minor, _micro = gtk.pygtk_version
    if ((_major < 2) and
        (_minor < 100) and #
        (_micro < 16)):
        print """
The PyGTK version you are using needs to be upgraded to
a newer version. Changes in the PyGTK code have required
the addition of some compatibility code in PythonCAD,
but these changes are temporary and will be removed in
a future PythonCAD release. Please upgrade your PyGTK
module to version 1.99.16 (at least), or retrieve and
build the PyGTK module from the CVS archive.
"""
    main()
